@@ -22,14 +22,18 @@
  *  along with GNU Make; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- */
+ ********************************************************************
+
+ last mod: $Id: ao_null.c 16982 2010-03-18 19:50:04Z xiphmont $
+
+ ********************************************************************/
 
 #include <stdio.h>
 #include <string.h>
 #include <ao/ao.h>
 
 static char *ao_null_options[] = {
-	"debug"
+  "debug","verbose","matrix","quiet"
 };
 static ao_info ao_null_info = {
 	AO_TYPE_LIVE,
@@ -40,13 +44,12 @@ static ao_info ao_null_info = {
 	AO_FMT_NATIVE,
 	0,
 	ao_null_options,
-	1
+	4
 };
 
 
 typedef struct ao_null_internal {
 	unsigned long byte_counter;
-	int debug_output;
 } ao_null_internal;
 
 
@@ -68,27 +71,23 @@ static int ao_null_device_init(ao_device *device)
 
 	internal = (ao_null_internal *) malloc(sizeof(ao_null_internal));
 
-	if (internal == NULL)	
+	if (internal == NULL)
 		return 0; /* Could not initialize device memory */
-	
+
 	internal->byte_counter = 0;
-	internal->debug_output = 0;
 
 	device->internal = internal;
+        device->output_matrix_order = AO_OUTPUT_MATRIX_FIXED;
 
 	return 1; /* Memory alloc successful */
 }
 
 
-static int ao_null_set_option(ao_device *device, const char *key, 
+static int ao_null_set_option(ao_device *device, const char *key,
 			      const char *value)
 {
 	ao_null_internal *internal = (ao_null_internal *) device->internal;
 
-	if (!strcmp(key, "debug")) {
-		internal->debug_output = 1;
-	}
-
 	return 1;
 }
 
@@ -99,11 +98,17 @@ static int ao_null_open(ao_device *device, ao_sample_format *format)
 	/* Use whatever format the client requested */
 	device->driver_byte_format = device->client_byte_format;
 
+        if(!device->inter_matrix){
+          /* by default, we want inter == in */
+          if(format->matrix)
+            device->inter_matrix = strdup(format->matrix);
+        }
+
 	return 1;
 }
 
 
-static int ao_null_play(ao_device *device, const char *output_samples, 
+static int ao_null_play(ao_device *device, const char *output_samples,
 			uint_32 num_bytes)
 {
 	ao_null_internal *internal = (ao_null_internal *)device->internal;
@@ -118,10 +123,7 @@ static int ao_null_close(ao_device *device)
 {
 	ao_null_internal *internal = (ao_null_internal *) device->internal;
 
-	if (internal->debug_output) {
-	  fprintf(stderr, "ao_null: %ld bytes sent to null device.\n",
-		  internal->byte_counter);
-	}
+	adebug("%ld bytes sent to null device.\n", internal->byte_counter);
 
 	return 1;
 }
