@@ -4,6 +4,7 @@
  *
  *      Original Copyright (C) Aaron Holtzman - May 1999
  *      Modifications Copyright (C) Stan Seibert - July 2000, July 2001
+ *                    Copyright (C) Monty - January 2010
  *
  *  This file is part of libao, a cross-platform audio output library.  See
  *  README for a history of this source code.
@@ -22,7 +23,11 @@
  *  along with GNU Make; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- */
+ ********************************************************************
+
+ last mod: $Id: ao_wav.c 16982 2010-03-18 19:50:04Z xiphmont $
+
+ ********************************************************************/
 
 
 #include <stdio.h>
@@ -31,12 +36,13 @@
 #include <signal.h>
 #include <ao/ao.h>
 
-#define WAVE_FORMAT_PCM  0x0001
-#define FORMAT_MULAW     0x0101
-#define IBM_FORMAT_ALAW  0x0102
-#define IBM_FORMAT_ADPCM 0x0103
+#define WAVE_FORMAT_PCM         0x0001
+#define FORMAT_MULAW            0x0101
+#define IBM_FORMAT_ALAW         0x0102
+#define IBM_FORMAT_ADPCM        0x0103
+#define WAVE_FORMAT_EXTENSIBLE  0xfffe
 
-#define WAV_HEADER_LEN 44
+#define WAV_HEADER_LEN 68
 
 #define WRITE_U32(buf, x) *(buf)     = (unsigned char)(x&0xff);\
 						  *((buf)+1) = (unsigned char)((x>>8)&0xff);\
@@ -55,23 +61,27 @@ struct riff_struct {
 };
 
 
-struct chunk_struct 
+struct chunk_struct
 {
 	unsigned char id[4];
 	unsigned int len;
 };
 
-struct common_struct 
+struct common_struct
 {
 	unsigned short wFormatTag;
 	unsigned short wChannels;
 	unsigned int dwSamplesPerSec;
 	unsigned int dwAvgBytesPerSec;
 	unsigned short wBlockAlign;
-	unsigned short wBitsPerSample;  /* Only for PCM */
+	unsigned short wBitsPerSample;
+        unsigned short cbSize;
+	unsigned short wValidBitsPerSample;
+        unsigned int   dwChannelMask;
+        unsigned short subFormat;
 };
 
-struct wave_header 
+struct wave_header
 {
 	struct riff_struct   riff;
 	struct chunk_struct  format;
@@ -80,6 +90,7 @@ struct wave_header
 };
 
 
+static char *ao_wav_options[] = {"matrix","verbose","quiet","debug"};
 static ao_info ao_wav_info =
 {
 	AO_TYPE_FILE,
@@ -89,8 +100,8 @@ static ao_info ao_wav_info =
 	"Sends output to a .wav file",
 	AO_FMT_LITTLE,
 	0,
-	NULL, /* No options */
-	0
+	ao_wav_options,
+        4
 };
 
 typedef struct ao_wav_internal
@@ -117,24 +128,25 @@ static int ao_wav_device_init(ao_device *device)
 
 	internal = (ao_wav_internal *) malloc(sizeof(ao_wav_internal));
 
-	if (internal == NULL)	
+	if (internal == NULL)
 		return 0; /* Could not initialize device memory */
-	
+
 	memset(&(internal->wave), 0, sizeof(internal->wave));
-	
+
 	device->internal = internal;
+        device->output_matrix = strdup("L,R,C,LFE,BL,BR,CL,CR,BC,SL,SR");
+        device->output_matrix_order = AO_OUTPUT_MATRIX_COLLAPSIBLE;
 
 	return 1; /* Memory alloc successful */
 }
 
 
-static int ao_wav_set_option(ao_device *device, const char *key, 
+static int ao_wav_set_option(ao_device *device, const char *key,
 			     const char *value)
 {
 	return 1; /* No options! */
 }
 
-
 static int ao_wav_open(ao_device *device, ao_sample_format *format)
 {
 	ao_wav_internal *internal = (ao_wav_internal *) device->internal;
@@ -142,8 +154,9 @@ static int ao_wav_open(ao_device *device, ao_sample_format *format)
 	int size = 0x7fffffff; /* Use a bogus size initially */
 
 	/* Store information */
-	internal->wave.common.wChannels = format->channels;
-	internal->wave.common.wBitsPerSample = format->bits;
+	internal->wave.common.wChannels = device->output_channels;
+	internal->wave.common.wBitsPerSample = ((format->bits+7)>>3)<<3;
+	internal->wave.common.wValidBitsPerSample = format->bits;
 	internal->wave.common.dwSamplesPerSec = format->rate;
 
 	memset(buf, 0, WAV_HEADER_LEN);
@@ -154,21 +167,24 @@ static int ao_wav_open(ao_device *device, ao_sample_format *format)
 	strncpy(internal->wave.riff.wave_id, "WAVE",4);
 
 	strncpy(internal->wave.format.id, "fmt ",4);
-	internal->wave.format.len = 16;
+	internal->wave.format.len = 40;
 
-	internal->wave.common.wFormatTag = WAVE_FORMAT_PCM;
-	internal->wave.common.dwAvgBytesPerSec = 
-		internal->wave.common.wChannels * 
+	internal->wave.common.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
+	internal->wave.common.dwAvgBytesPerSec =
+		internal->wave.common.wChannels *
 		internal->wave.common.dwSamplesPerSec *
 		(internal->wave.common.wBitsPerSample >> 3);
 
-	internal->wave.common.wBlockAlign = 
-		internal->wave.common.wChannels * 
+	internal->wave.common.wBlockAlign =
+		internal->wave.common.wChannels *
 		(internal->wave.common.wBitsPerSample >> 3);
+	internal->wave.common.cbSize = 22;
+	internal->wave.common.subFormat = WAVE_FORMAT_PCM;
+        internal->wave.common.dwChannelMask=device->output_mask;
 
 	strncpy(internal->wave.data.id, "data",4);
 
-	internal->wave.data.len = size - 44;
+	internal->wave.data.len = size - WAV_HEADER_LEN;
 
 	strncpy(buf, internal->wave.riff.id, 4);
 	WRITE_U32(buf+4, internal->wave.riff.len);
@@ -181,10 +197,15 @@ static int ao_wav_open(ao_device *device, ao_sample_format *format)
 	WRITE_U32(buf+28, internal->wave.common.dwAvgBytesPerSec);
 	WRITE_U16(buf+32, internal->wave.common.wBlockAlign);
 	WRITE_U16(buf+34, internal->wave.common.wBitsPerSample);
-	strncpy(buf+36, internal->wave.data.id, 4);
-	WRITE_U32(buf+40, internal->wave.data.len);
+	WRITE_U16(buf+36, internal->wave.common.cbSize);
+	WRITE_U16(buf+38, internal->wave.common.wValidBitsPerSample);
+	WRITE_U32(buf+40, internal->wave.common.dwChannelMask);
+	WRITE_U16(buf+44, internal->wave.common.subFormat);
+        memcpy(buf+46,"\x00\x00\x00\x00\x10\x00\x80\x00\x00\xAA\x00\x38\x9B\x71",14);
+	strncpy(buf+60, internal->wave.data.id, 4);
+	WRITE_U32(buf+64, internal->wave.data.len);
 
-	if (fwrite(buf, sizeof(char), WAV_HEADER_LEN, device->file) 
+	if (fwrite(buf, sizeof(char), WAV_HEADER_LEN, device->file)
 	    != WAV_HEADER_LEN) {
 		return 0; /* Could not write wav header */
 	}
@@ -198,15 +219,15 @@ static int ao_wav_open(ao_device *device, ao_sample_format *format)
 /*
  * play the sample to the already opened file descriptor
  */
-static int ao_wav_play(ao_device *device, const char *output_samples, 
+static int ao_wav_play(ao_device *device, const char *output_samples,
 			uint_32 num_bytes)
 {
-	if (fwrite(output_samples, sizeof(char), num_bytes, 
+	if (fwrite(output_samples, sizeof(char), num_bytes,
 		   device->file) < num_bytes)
 		return 0;
 	else
 		return 1;
-	
+
 }
 
 static int ao_wav_close(ao_device *device)
@@ -226,26 +247,26 @@ static int ao_wav_close(ao_device *device)
 	/* Go back and set correct length info */
 
 	internal->wave.riff.len = size - 8;
-	internal->wave.data.len = size - 44;
+	internal->wave.data.len = size - WAV_HEADER_LEN;
 
 	/* Rewind to riff len and write it */
 	if (fseek(device->file, 4, SEEK_SET) < 0)
 		return 0; /* Wav header corrupt */
-	
+
 	WRITE_U32(buf, internal->wave.riff.len);
 	if (fwrite(buf, sizeof(char), 4, device->file) < 4)
 	  return 0; /* Wav header corrupt */
 
 
 	/* Rewind to data len and write it */
-	if (fseek(device->file, 40, SEEK_SET) < 0)
+	if (fseek(device->file, 64, SEEK_SET) < 0)
 		return 0; /* Wav header corrupt */
-	
+
 	WRITE_U32(buf, internal->wave.data.len);
 	if (fwrite(buf, sizeof(char), 4, device->file) < 4)
 	  return 0; /* Wav header corrupt */
 
-       
+
 	return 1; /* Wav header correct */
 }
 
@@ -267,4 +288,3 @@ ao_functions ao_wav = {
 	ao_wav_close,
 	ao_wav_device_clear
 };
-
