@@ -23,7 +23,11 @@
  *  along with GNU Make; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- */
+ ********************************************************************
+
+ last mod: $Id: ao_irix.c 16982 2010-03-18 19:50:04Z xiphmont $
+
+ ********************************************************************/
 
 #include <stdio.h>
 #include <errno.h>
@@ -45,6 +49,7 @@ typedef struct ao_irix_internal {
 	int channels;
 } ao_irix_internal;
 
+static char *ao_irix_options[] = {"matrix","verbose","quiet","debug"};
 
 static ao_info ao_irix_info =
 {
@@ -55,8 +60,8 @@ static ao_info ao_irix_info =
 	"Outputs to the IRIX Audio Library.",
 	AO_FMT_NATIVE,
 	20,
-	NULL,
-	1
+	ao_irix_options,
+	4
 };
 
 int ao_plugin_test(void)
@@ -95,6 +100,7 @@ int ao_plugin_device_init(ao_device *device)
 	internal->channels = 2;
 
 	device->internal = internal;
+        device->output_matrix_order = AO_OUTPUT_MATRIX_FIXED;
 
 	/* Device-specific initialization was successful. */
 	return 1;
@@ -114,27 +120,27 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 	int  wsize = AL_SAMPLE_16;
 
 	if (alSetQueueSize(internal->alconfig, AO_IRIX_BUFFER_SIZE) < 0) {
-		fprintf(stderr, "alSetQueueSize failed: %s\n",
+		aerror("alSetQueueSize failed: %s\n",
 			alGetErrorString(oserror()));
 		return 0;
 	}
 
-	if (alSetChannels(internal->alconfig, format->channels) < 0) {
-		fprintf(stderr, "alSetChannels(%d) failed: %s\n",
-			format->channels, alGetErrorString(oserror()));
+	if (alSetChannels(internal->alconfig, device->output_channels) < 0) {
+		aerror("alSetChannels(%d) failed: %s\n",
+			device->output_channels, alGetErrorString(oserror()));
 		return 0;
 	}
 
-	internal->channels = format->channels;
+	internal->channels = device->output_channels;
 
 	if (alSetDevice(internal->alconfig, dev) < 0) {
-		fprintf(stderr, "alSetDevice failed: %s\n",
+		aerror("alSetDevice failed: %s\n",
 			alGetErrorString(oserror()));
 		return 0;
 	}
 
 	if (alSetSampFmt(internal->alconfig, AL_SAMPFMT_TWOSCOMP) < 0) {
-		fprintf(stderr, "alSetSampFmt failed: %s\n",
+		aerror("alSetSampFmt failed: %s\n",
 			alGetErrorString(oserror()));
 		return 0;
 	}
@@ -156,13 +162,13 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 		break;
 
 	default:
-		fprintf(stderr, "Irix audio: unsupported bit with %d\n",
+		aerror("unsupported bit with %d\n",
 			format->bits);
 		break;
 	}
 
 	if (alSetWidth(internal->alconfig, wsize) < 0) {
-		fprintf(stderr, "alSetWidth failed: %s\n",
+		aerror("alSetWidth failed: %s\n",
 			alGetErrorString(oserror()));
 		alClosePort(internal->alport);
 		return 0;
@@ -171,7 +177,7 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 	internal->alport = alOpenPort("libao", "w", internal->alconfig);
 	if (internal->alport == NULL)
 	{
-		fprintf(stderr, "alOpenPort failed: %s\n",
+		aerror("alOpenPort failed: %s\n",
 			alGetErrorString(oserror()));
 		return 0;
 	}
@@ -180,12 +186,18 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 	params.value.ll = alDoubleToFixed((double) format->rate);
 	if (alSetParams(dev, &params, 1) < 0)
 	{
-		printf("alSetParams() failed: %s\n", alGetErrorString(oserror()));
+		aerror("alSetParams() failed: %s\n", alGetErrorString(oserror()));
 		alClosePort(internal->alport);
 		return 0;
 	}
 
 	device->driver_byte_format = AO_FMT_NATIVE;
+        if(!device->inter_matrix){
+          /* set up matrix such that users are warned about > stereo playback */
+          if(device->output_channels<=2)
+            device->inter_matrix=strdup("L,R");
+          //else no matrix, which results in a warning
+        }
 
 	return 1;
 }
