@@ -22,7 +22,11 @@
  *  along with GNU Make; see the file COPYING.  If not, write to
  *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- */
+ ********************************************************************
+
+ last mod: $Id: ao_oss.c 17000 2010-03-23 06:15:37Z xiphmont $
+
+ ********************************************************************/
 
 #include <stdio.h>
 #include <errno.h>
@@ -41,7 +45,7 @@
 #include "ao/plugin.h"
 
 
-static char *ao_oss_options[] = {"dsp"};
+static char *ao_oss_options[] = {"dsp","verbose","quiet","matrix","debug"};
 static ao_info ao_oss_info =
 {
 	AO_TYPE_LIVE,
@@ -52,7 +56,7 @@ static ao_info ao_oss_info =
 	AO_FMT_NATIVE,
 	20,
 	ao_oss_options,
-	1
+	5
 };
 
 
@@ -78,11 +82,11 @@ typedef struct ao_oss_internal {
 int _open_default_oss_device (char **dev_path, int blocking)
 {
 	int fd;
-	char *err = NULL;
-	char *dev = NULL;
 
 	/* default: first try the devfs path */
-	*dev_path = strdup("/dev/sound/dsp");
+	if(!(*dev_path = strdup("/dev/sound/dsp")))
+          return -1;
+
 #ifdef BROKEN_OSS
 	fd = open(*dev_path, O_WRONLY | O_NONBLOCK);
 #else
@@ -90,13 +94,12 @@ int _open_default_oss_device (char **dev_path, int blocking)
 #endif /* BROKEN_OSS */
 
 	/* then try the original dsp path */
-	if(fd < 0) 
+	if(fd < 0)
 	{
 		/* no? then try the traditional path */
-		err = strdup(strerror(errno));
-		dev = strdup(*dev_path);
 		free(*dev_path);
-		*dev_path = strdup("/dev/dsp");
+		if(!(*dev_path = strdup("/dev/dsp")))
+                  return -1;
 #ifdef BROKEN_OSS
 		fd = open(*dev_path, O_WRONLY | O_NONBLOCK);
 #else
@@ -117,21 +120,13 @@ int _open_default_oss_device (char **dev_path, int blocking)
 #endif /* BROKEN_OSS */
 
 	/* Deal with error cases */
-	if(fd < 0) 
+	if(fd < 0)
 	{
-	  /*			fprintf(stderr,
-				"libao - error: Could not open either default device:\n"
-				"  %s - %s\n"
-				"  %s - %s\n",
-				err, dev,
-				strerror(errno), *dev_path); */
+                /* could not open either default device */
 		free(*dev_path);
 		*dev_path = NULL;
 	}
 
-	if (err) free(err);
-	if (dev) free(dev);
-
 	return fd;
 }
 
@@ -143,7 +138,7 @@ int ao_plugin_test()
 
 	/* OSS emulation in ALSA will by default cause the open() call
 	   to block if the dsp is in use.  This will freeze the default
-	   driver detection unless the O_NONBLOCK flag is passed to 
+	   driver detection unless the O_NONBLOCK flag is passed to
 	   open().  We cannot use this flag when we actually open the
 	   device for writing because then we will overflow the buffer. */
 	if ( (fd = _open_default_oss_device(&dev_path, 0)) < 0 )
@@ -168,12 +163,13 @@ int ao_plugin_device_init(ao_device *device)
 
 	internal = (ao_oss_internal *) malloc(sizeof(ao_oss_internal));
 
-	if (internal == NULL)	
+	if (internal == NULL)
 		return 0; /* Could not initialize device memory */
-	
+
 	internal->dev = NULL;
-	
+
 	device->internal = internal;
+        device->output_matrix_order = AO_OUTPUT_MATRIX_FIXED;
 
 	return 1; /* Memory alloc successful */
 }
@@ -185,8 +181,9 @@ int ao_plugin_set_option(ao_device *device, const char *key, const char *value)
 
 	if (!strcmp(key, "dsp")) {
 		/* Free old string in case "dsp" set twice in options */
-		free(internal->dev); 
-		internal->dev = strdup(value);
+		free(internal->dev);
+		if(!(internal->dev = strdup(value)))
+                  return 1;
 	}
 
 	return 1;
@@ -202,42 +199,54 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 	int tmp;
 
 	/* Open the device driver */
-	
+
 	if (internal->dev != NULL) {
 		/* open the user-specified path */
 		internal->fd = open(internal->dev, O_WRONLY);
 
 		if(internal->fd < 0) {
-		  /* fprintf(stderr,"libao - %s: Opening audio device %s\n",
-		     strerror(errno), internal->dev); */
-			return 0;  /* Cannot open device */
+                  aerror("open(%s) => %s\n",internal->dev,strerror(errno));
+                  return 0;  /* Cannot open device */
 		}
 
 	} else {
 		internal->fd = _open_default_oss_device(&internal->dev, 1);
-		if (internal->fd < 0)
-			return 0;  /* Cannot open default device */
+		if (internal->fd < 0){
+                  aerror("open default => %s\n",strerror(errno));
+                  return 0;  /* Cannot open default device */
+                }
 	}
 
 	/* Now set all of the parameters */
 
-	switch (format->channels)
+#ifdef SNDCTL_DSP_CHANNELS
+        /* OSS versions > 2 */
+        tmp = device->output_channels;
+        if (ioctl(internal->fd,SNDCTL_DSP_CHANNELS,&tmp) < 0 ||
+            tmp != device->output_channels) {
+          aerror("cannot set channels to %d\n", device->output_channels);
+          goto ERR;
+        }
+#else
+        /* OSS versions < 4 */
+	switch (device->output_channels)
 	{
 	case 1: tmp = 0;
 		break;
 	case 2: tmp = 1;
 		break;
-	default:fprintf(stderr,"libao - Unsupported number of channels: %d.",
-			format->channels);
+	default:aerror("Unsupported number of channels: %d.",
+                       device->output_channels);
 		goto ERR;
 	}
-	
-	if (ioctl(internal->fd,SNDCTL_DSP_STEREO,&tmp) < 0 || 
-			tmp+1 != format->channels) {
-		fprintf(stderr, "libao - OSS cannot set channels to %d\n", 
-			format->channels);
-		goto ERR;
+
+	if (ioctl(internal->fd,SNDCTL_DSP_STEREO,&tmp) < 0 ||
+			tmp+1 != device->output_channels) {
+          aerror("cannot set channels to %d\n",
+                 device->output_channels);
+          goto ERR;
 	}
+#endif
 
 	/* To eliminate the need for a swap buffer, we set the device
 	   to use whatever byte format the client selected. */
@@ -245,17 +254,17 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 	{
 	case 8: tmp = AFMT_S8;
 		break;
-        case 16: tmp = device->client_byte_format == AO_FMT_BIG ? 
+        case 16: tmp = device->client_byte_format == AO_FMT_BIG ?
 		   AFMT_S16_BE : AFMT_S16_LE;
 	        device->driver_byte_format = device->client_byte_format;
 	        break;
-	default:fprintf(stderr,"libao - Unsupported number of bits: %d.",
+	default:aerror("Unsupported number of bits: %d.",
 			format->bits);
 		goto ERR;
 	}
 
 	if (ioctl(internal->fd,SNDCTL_DSP_SAMPLESIZE,&tmp) < 0) {
-		fprintf(stderr, "libao - OSS cannot set sample size to %d\n",
+		aerror("cannot set sample size to %d\n",
 			format->bits);
 		goto ERR;
 	}
@@ -266,7 +275,7 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 	   of whack. */
 	if (ioctl(internal->fd,SNDCTL_DSP_SPEED, &tmp) < 0
 	    || tmp > 1.02 * format->rate || tmp < 0.98 * format->rate) {
-		fprintf(stderr, "libao - OSS cannot set rate to %d\n", 
+		aerror("cannot set rate to %d\n",
 			format->rate);
 		goto ERR;
 	}
@@ -277,11 +286,23 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 				&(internal->buf_size)) < 0) ||
 			internal->buf_size<=0 )
 	{
-		fprintf(stderr, "libao - OSS cannot get buffer size for "
-				" device\n");
-		goto ERR;
+          /* Some versions of the *BSD OSS drivers use a subtly
+             different SNDCTL_DSP_GETBLKSIZE ioctl implementation
+             which is, oddly, incompatible with the shipped
+             declaration in soundcard.h.  This ioctl isn't necessary
+             anyway, it's just tuning.  Soldier on without, */
+
+          adebug("cannot get buffer size for device; using a default of 1024kB\n");
+          internal->buf_size=1024;
 	}
 
+        if(!device->inter_matrix){
+          /* set up matrix such that users are warned about > stereo playback */
+          if(device->output_channels<=2)
+            device->inter_matrix=strdup("L,R");
+          //else no matrix, which results in a warning
+        }
+
 	return 1; /* Open successful */
 
  ERR:
@@ -292,7 +313,7 @@ int ao_plugin_open(ao_device *device, ao_sample_format *format)
 /*
  * play the sample to the already opened file descriptor
  */
-int ao_plugin_play(ao_device *device, const char *output_samples, 
+int ao_plugin_play(ao_device *device, const char *output_samples,
 		uint_32 num_bytes)
 {
 	int ret;
@@ -300,15 +321,16 @@ int ao_plugin_play(ao_device *device, const char *output_samples,
 	ao_oss_internal *internal = (ao_oss_internal *) device->internal;
 
 	while(num_bytes > 0) {
-		send = num_bytes>internal->buf_size?
-			internal->buf_size:num_bytes;
-		ret = write(internal->fd, output_samples, send);
+          send = num_bytes>internal->buf_size?
+            internal->buf_size:num_bytes;
+          ret = write(internal->fd, output_samples, send);
 
-		if (ret <= 0)
-			return 0;
-
-		num_bytes-=ret;
-		output_samples+=ret;
+          if (ret < 0){
+            if(errno == EINTR) continue;
+            return 0;
+          }
+          num_bytes-=ret;
+          output_samples+=ret;
 	}
 
 	return 1;
